using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace Dronken_Kerstman
{
    public class Game : Microsoft.Xna.Framework.Game
    {
        Random ran = new Random(Guid.NewGuid().GetHashCode());
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        SpriteFont font;

        public static int screenWidth = 640;
        public static int screenHeight = screenWidth;
        public static int boardWidth = (screenWidth / 4) * 3;
        public static int boardHeight = (screenHeight / 4) * 3;
        public static int boardX = (screenWidth / 2) - (boardWidth / 2);
        public static int boardY = (screenHeight / 2) - (boardHeight / 2);
        public static int tick = 20;
        public static bool fullscreen = false;

        public int drunk = 30;
        public bool dead = false;
        public float loop = 0f;

        public Sprite roof = new Sprite();
        public Santa santa = new Santa();
        public Beer beer;
        public Song s1, s2, fade;
        public Color trippy = Color.White;
        public Color back = Color.CornflowerBlue;


        public Game()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "RedBull";
        }

        protected override void Initialize()
        {
            graphics.PreferredBackBufferWidth = screenWidth;
            graphics.PreferredBackBufferHeight = screenHeight;
            graphics.IsFullScreen = fullscreen;
            graphics.ApplyChanges();

            base.Initialize();
        }

        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);

            roof.tex = emptyTex(GraphicsDevice, boardWidth, boardHeight, Color.Yellow);
            roof.pos = new Vector2(boardX, boardY);

            santa.tex = Content.Load<Texture2D>("santa");//emptyTex(GraphicsDevice, 10, 20, Color.Beige);
            santa.pos = new Vector2((screenWidth / 2) - (santa.tex.Width / 2), (screenHeight / 2) - (santa.tex.Width / 2));

            this.beer = new Beer(Content.Load<Texture2D>("can"));
            this.font = Content.Load<SpriteFont>("font");
            this.s1 = Content.Load<Song>("m\\1");
            this.s2 = Content.Load<Song>("m\\2");
            this.fade = Content.Load<Song>("m\\fade");
            MediaPlayer.Play(s1);
            MediaPlayer.IsRepeating = true;
        }

        bool doneCheck = false;
        bool playing = false;
        bool allowTrippy = false;
        protected override void Update(GameTime gameTime)
        {
            santa.update(gameTime); // You can't quit without him!

            if (!dead)
            {
                if (Beer.intersects(beer, santa))
                {
                    drunk++;
                    beer.pos = Beer.newPos(beer.tex.Width, beer.tex.Height);
                }

                if (santa.pos.X > roof.pos.X + roof.tex.Width || santa.pos.X + santa.tex.Width < roof.pos.X ||
                    santa.pos.Y > roof.pos.Y + roof.tex.Height || santa.pos.Y + santa.tex.Height < roof.pos.Y) dead = true;

                loop += gameTime.ElapsedGameTime.Milliseconds;

                if (loop >= tick)
                {
                    santa.pos.X += random();
                    santa.pos.Y += random(); // Prevent that santa is going "down"
                    loop = 0f;
                }

            }
            else if (drunk > 9)
            {
                santa.pos = Beer.newPos(santa.tex.Width / 3, santa.tex.Height);
            }

            if (drunk > 9)
            {
                if (!doneCheck)
                {
                    MediaPlayer.IsRepeating = false;
                    MediaPlayer.Play(fade);
                    doneCheck = true;
                    playing = true;
                }
                else if (playing)
                {
                    if (MediaPlayer.State == MediaState.Stopped)
                    {
                        playing = false;
                        MediaPlayer.Play(s2);
                        MediaPlayer.IsRepeating = true;
                        allowTrippy = true;
                    }
                }

                if (allowTrippy)
                {
                    back = trippy;
                    trippy = new Color(ran.Next(0, 256), ran.Next(0, 256), ran.Next(0, 256));
                }
            }

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(back);

            spriteBatch.Begin();

            if (!dead)
            {
                spriteBatch.DrawString(font, "Score: " + drunk, new Vector2(30, 30), Color.White);
                spriteBatch.Draw(roof.tex, roof.pos, trippy);
                beer.draw(spriteBatch);
            }
            else
            {
                spriteBatch.DrawString(font, "You are dead!", new Vector2(screenWidth / 2 - 200, screenHeight / 2 - 65), 
                    Color.Lerp(Color.MediumVioletRed, Color.Pink, 0.15f), 0f, Vector2.Zero, 3f, SpriteEffects.None, 0f);
                spriteBatch.DrawString(font, "Score: " + drunk, new Vector2(screenWidth / 2 - 60, screenHeight / 2), Color.White);
            }

            santa.draw(spriteBatch); // special effects!
            spriteBatch.End();

            base.Draw(gameTime);
        }

        public static Texture2D emptyTex(GraphicsDevice dev, int width, int height, Color color)
        {
            Texture2D tex = new Texture2D(dev, width, height);
            Color[] clr = new Color[width * height];
            for (int i = 0; i < clr.Length; i++) clr[i] = color;
            tex.SetData(clr);
            return tex;
        }

        public int random(double d = 0.5)
        {
            this.ran = new Random(ran.Next(0, 34145560));

            if (ran.Next(0, int.MaxValue) > int.MaxValue / 2)
            {
                return ran.Next(0, drunk);
            }
            else
            {
                return -ran.Next(0, drunk);
            }            
        }
    }
}
